"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SolanaPdaQueryResponse = exports.SolanaPdaQueryRequest = exports.SolanaMaxSeedLen = exports.SolanaMaxSeeds = void 0;
const buffer_1 = require("buffer");
const BinaryWriter_1 = require("./BinaryWriter");
const request_1 = require("./request");
const utils_1 = require("./utils");
const BinaryReader_1 = require("./BinaryReader");
// According to the spec, there may be at most 16 seeds.
// https://github.com/gagliardetto/solana-go/blob/6fe3aea02e3660d620433444df033fc3fe6e64c1/keys.go#L559
exports.SolanaMaxSeeds = 16;
// According to the spec, a seed may be at most 32 bytes.
// https://github.com/gagliardetto/solana-go/blob/6fe3aea02e3660d620433444df033fc3fe6e64c1/keys.go#L557
exports.SolanaMaxSeedLen = 32;
class SolanaPdaQueryRequest {
    constructor(commitment, pdas, minContextSlot, dataSliceOffset, dataSliceLength) {
        this.pdas = pdas;
        pdas.forEach((pda) => {
            if (pda.programAddress.length != 32) {
                throw new Error(`Invalid program address, must be 32 bytes: ${pda.programAddress}`);
            }
            if (pda.seeds.length == 0) {
                throw new Error(`Invalid pda, has no seeds: ${buffer_1.Buffer.from(pda.programAddress).toString("hex")}`);
            }
            if (pda.seeds.length > exports.SolanaMaxSeeds) {
                throw new Error(`Invalid pda, has too many seeds: ${buffer_1.Buffer.from(pda.programAddress).toString("hex")}`);
            }
            pda.seeds.forEach((seed) => {
                if (seed.length == 0) {
                    throw new Error(`Invalid pda, seed is null: ${buffer_1.Buffer.from(pda.programAddress).toString("hex")}`);
                }
                if (seed.length > exports.SolanaMaxSeedLen) {
                    throw new Error(`Invalid pda, seed is too long: ${buffer_1.Buffer.from(pda.programAddress).toString("hex")}`);
                }
            });
        });
        this.commitment = commitment;
        this.minContextSlot = (0, utils_1.bigIntWithDef)(minContextSlot);
        this.dataSliceOffset = (0, utils_1.bigIntWithDef)(dataSliceOffset);
        this.dataSliceLength = (0, utils_1.bigIntWithDef)(dataSliceLength);
    }
    type() {
        return request_1.ChainQueryType.SolanaPda;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint32(this.commitment.length)
            .writeUint8Array(buffer_1.Buffer.from(this.commitment))
            .writeUint64(this.minContextSlot)
            .writeUint64(this.dataSliceOffset)
            .writeUint64(this.dataSliceLength)
            .writeUint8(this.pdas.length);
        this.pdas.forEach((pda) => {
            writer.writeUint8Array(pda.programAddress).writeUint8(pda.seeds.length);
            pda.seeds.forEach((seed) => {
                writer.writeUint32(seed.length).writeUint8Array(seed);
            });
        });
        return writer.data();
    }
    static from(bytes) {
        const reader = new BinaryReader_1.BinaryReader((0, utils_1.coalesceUint8Array)(bytes));
        return this.fromReader(reader);
    }
    static fromReader(reader) {
        const commitmentLength = reader.readUint32();
        const commitment = reader.readString(commitmentLength);
        if (commitment !== "finalized") {
            throw new Error(`Invalid commitment: ${commitment}`);
        }
        const minContextSlot = reader.readUint64();
        const dataSliceOffset = reader.readUint64();
        const dataSliceLength = reader.readUint64();
        const numPdas = reader.readUint8();
        const pdas = [];
        for (let idx = 0; idx < numPdas; idx++) {
            const programAddress = reader.readUint8Array(32);
            let seeds = [];
            const numSeeds = reader.readUint8();
            for (let idx2 = 0; idx2 < numSeeds; idx2++) {
                const seedLen = reader.readUint32();
                const seed = reader.readUint8Array(seedLen);
                seeds.push(seed);
            }
            pdas.push({ programAddress, seeds });
        }
        return new SolanaPdaQueryRequest(commitment, pdas, minContextSlot, dataSliceOffset, dataSliceLength);
    }
}
exports.SolanaPdaQueryRequest = SolanaPdaQueryRequest;
class SolanaPdaQueryResponse {
    constructor(slotNumber, blockTime, blockHash, results) {
        if (blockHash.length != 32) {
            throw new Error(`Invalid block hash, should be 32 bytes long: ${blockHash}`);
        }
        for (const result of results) {
            if (result.account.length != 32) {
                throw new Error(`Invalid account, should be 32 bytes long: ${result.account}`);
            }
            if (result.owner.length != 32) {
                throw new Error(`Invalid owner, should be 32 bytes long: ${result.owner}`);
            }
        }
        this.slotNumber = slotNumber;
        this.blockTime = blockTime;
        this.blockHash = blockHash;
        this.results = results;
    }
    type() {
        return request_1.ChainQueryType.SolanaPda;
    }
    serialize() {
        const writer = new BinaryWriter_1.BinaryWriter()
            .writeUint64(this.slotNumber)
            .writeUint64(this.blockTime)
            .writeUint8Array(this.blockHash)
            .writeUint8(this.results.length);
        for (const result of this.results) {
            writer
                .writeUint8Array(result.account)
                .writeUint8(result.bump)
                .writeUint64(result.lamports)
                .writeUint64(result.rentEpoch)
                .writeUint8(result.executable ? 1 : 0)
                .writeUint8Array(result.owner)
                .writeUint32(result.data.length)
                .writeUint8Array(result.data);
        }
        return writer.data();
    }
    static from(bytes) {
        const reader = new BinaryReader_1.BinaryReader((0, utils_1.coalesceUint8Array)(bytes));
        return this.fromReader(reader);
    }
    static fromReader(reader) {
        const slotNumber = reader.readUint64();
        const blockTime = reader.readUint64();
        const blockHash = reader.readUint8Array(32);
        const resultsLength = reader.readUint8();
        const results = [];
        for (let idx = 0; idx < resultsLength; idx++) {
            const account = reader.readUint8Array(32);
            const bump = reader.readUint8();
            const lamports = reader.readUint64();
            const rentEpoch = reader.readUint64();
            const executableU8 = reader.readUint8();
            const executable = executableU8 != 0;
            const owner = reader.readUint8Array(32);
            const dataLength = reader.readUint32();
            const data = reader.readUint8Array(dataLength);
            results.push({
                account,
                bump,
                lamports,
                rentEpoch,
                executable,
                owner,
                data,
            });
        }
        return new SolanaPdaQueryResponse(slotNumber, blockTime, blockHash, results);
    }
}
exports.SolanaPdaQueryResponse = SolanaPdaQueryResponse;
